# -*- coding: utf-8 -*-

"""
flask_via.examples.include.foo.routes
=====================================

A include ``Flask-Via`` example Flask application.
"""

from flask_via.examples.include.foo.views import BarView, BazView
from flask.ext.via.routers import Include
from flask.ext.via.routers.default import Pluggable

routes = [
    Pluggable('/bar', BarView, 'bar'),
    Pluggable('/baz', BazView, 'baz'),
    Include(
        'flask_via.examples.include.bar.routes',
        url_prefix='/bar',
        endpoint='bar')
]



# -*- coding: utf-8 -*-

"""
flask_via.routers.restful
-------------------------

Routers for the Flask-Restful framework.
"""

from flask_via.routers import BaseRouter


class Resource(BaseRouter):
    """ The Resource router allows you to define Flask-Restful routes and have
    those API resources added to the application automatically. For this to
    work you must at ``init_app`` time pass a optional keyword argument
    ``restful_api`` to ``init_app`` with its value being the restful api
    extension instance.

    .. versionadded:: 2014.05.06

    Example
    -------
    .. sourcecode:: python

        app = Flask(__name__)
        api = restful.Api(app)

        class FooResource(restful.Resource):

            def get(self):
                return {'hello': 'world'}

        routes = [
            Resource('/foo', FooResource)
        ]

        via = Via()
        via.init_app(
            app,
            routes_module='flask_via.examples.restful',
            restful_api=api)

        if __name__ == '__main__':
            app.run(debug=True)

    """

    def __init__(self, url, resource, endpoint=None):
        """ Constructor for flask restful resource router.

        Arguments
        ---------
        url : str
            The url to use for the route
        resource
            A flask ``restful.Resource`` resource class

        Keyword Arguments
        -----------------
        endpoint : str, optional
            Optional, override ``Flask-Restful`` automatic endpoint naming
        """

        self.url = url
        self.resource = resource
        self.endpoint = endpoint

    def add_to_app(self, app, **kwargs):
        """ Adds the restul api resource route to the application.

        Arguments
        ---------
        app : flask.app.Flask
            Flask application instance, this is ignored.
        \*\*kwargs
            Arbitrary keyword arguments

        Raises
        ------
        NotImplementedError
            If ``restful_api`` is not provided
        """

        try:
            restful_api = kwargs['restful_api']
        except KeyError:
            raise NotImplementedError(
                'restful_api not passed to add_to_app, did you add it to '
                'via.init_app?')

        restful_api.add_resource(
            self.resource,
            self.url,
            endpoint=self.endpoint)



# -*- coding: utf-8 -*-

"""
flask_via.exceptions
--------------------

Custom exceptions which can be thrown by ``Flask-Via``.
"""


class ImproperlyConfigured(Exception):
    """ Raised in the event ``Flask-Via`` has not been properly configured

    .. versionadded: 2014.05.19
    """

    pass



# -*- coding: utf-8 -*-

"""
flask_via.routers.default
-------------------------

A set of flask specific router classes to be used when defining routes.

Example
-------

.. sourcecode:: python

    from flask.ext.via.routes.flask import Basic, Pluggable
    from yourapp.views import BarView, foo_view

    routes = [
        Basic('/foo', 'foo', foo_view),
        Pluggable('/bar', BarView, 'bar'),
    ]

"""

import warnings

from flask import Blueprint as FlaskBlueprint
from flask_via import RoutesImporter
from flask_via.routers import BaseRouter


class Functional(BaseRouter):
    """ A basic Flask router, used for the most basic form of flask routes,
    namely functionally based views which would normally use the ``@route``
    decorator.

    .. versionadded:: 2014.05.19

    Example
    -------
    .. sourcecode:: python

        from flask.ext.via.routes import default
        from yourapp.views import foo_view, bar_view

        routes = [
            default.Functional('/foo', 'foo', foo_view),
            default.Functional('/bar', 'bar', bar_view),
        ]
    """

    def __init__(self, url, func, endpoint=None):
        """ Basic router constructor, stores passed arguments on the
        instance.

        Arguments
        ---------
        url : str
            The url to use for the route
        func : function
            The view function to connect the route with

        Keyword Arguments
        -----------------
        endpoint : str, optional
            Optional endpoint string, by default flask will use the
            view function name as the endpoint name, use this argument
            to change the endpoint name.
        """

        self.url = url
        self.func = func
        self.endpoint = endpoint

    def add_to_app(self, app, **kwargs):
        """ Adds the url route to the flask application object.mro

        .. versionchanged:: 2014.05.08

            * ``url_prefix`` can now be prefixed if present in kwargs

        .. versionchanged:: 2014.05.19

            * ``endpoint`` can now be prefixed if present in kwargs

        Arguments
        ---------
        app : flask.app.Flask
            Flask application instance
        \*\*kwargs
            Arbitrary keyword arguments passed in to ``init_app``
        """

        url = self.url
        endpoint = self.endpoint

        #: If this route was included a url preifx may have been passed
        #: to the route
        if 'url_prefix' in kwargs:
            url = kwargs['url_prefix'] + url

        #: If this route was included a endpoint prefix may have been passed
        #: to the route
        if 'endpoint' in kwargs:
            if endpoint is None:
                endpoint = self.func.__name__
            endpoint = kwargs['endpoint'] + endpoint

        try:
            app.add_url_rule(url, endpoint, self.func)
        except AssertionError:
            # TODO: Log / Warn
            pass


class Basic(Functional):
    """ This is deprecated and will be removed in the next release. Please use
    :class:`.Functional`.

    .. versionadded:: 2014.05.06
    .. deprecated:: 2014.05.19
    """

    def __init__(self, *args, **kwargs):
        """ Issues ``DeprecationWarning`` as this is deprecated and will be
        removed in a later version.
        """

        warnings.simplefilter('always', DeprecationWarning)
        warnings.warn(
            'Basic router will be removed in a future version, please switch '
            'to the Functional router.',
            DeprecationWarning)

        super(Basic, self).__init__(*args, **kwargs)


class Pluggable(BaseRouter):
    """ Pluggable View router class, allows Flask pluggable view routes to be
    added to the flask application.

    .. versionadded:: 2014.05.06

    Example
    -------
    .. sourcecode:: python

        from flask.ext.via.routers import flask
        from flask.views import MethodView

        class FooView(MethodView):
            def get(self):
                return 'foo view'

        class BarView(MethodView):
            def get(self):
                return 'bar view'

        routes = [
            flask.Pluggable('/', FooView, 'foo')
            flask.Pluggable('/', BarView, 'bar')
        ]
    """

    def __init__(self, url, view, endpoint, **kwargs):
        """ Pluggable router constructor, stores passed arguments on instance.

        .. versionchanged:: 2014.05.19

            * Added ``view`` argument
            * Added ``endpoint`` argument

        Arguments
        ---------
        url : str
            The url to use for the route
        view : class
            The Flask pluggable view class, for example:
            * :class:`flask.views.View`
            * :class:`flask.views.MethodView`
        endpoint : str
            The Flask endpoint name for the view, this is required for Flask
            pluggable views.
        \*\*kwargs :
            Arbitrary keyword arguments for ``add_url_rule``
        """

        self.url = url
        self.view = view
        self.endpoint = endpoint
        self.kwargs = kwargs

    def add_to_app(self, app, **kwargs):
        """ Adds the url route to the flask application object.

        .. versionchanged:: 2014.05.19

            Updated ``add_url_rule`` to support endpoint prefixing and support
            new way of defining Pluggable views

        Arguments
        ---------
        app : flask.app.Flask
            Flask application instance
        \*\*kwargs
            Arbitrary keyword arguments passed in to ``init_app``
        """

        url = self.url
        endpoint = self.endpoint

        #: If this route was included a url preifx may have been passed
        #: to the route
        if 'url_prefix' in kwargs:
            url = kwargs['url_prefix'] + url

        #: If this route was included a endpoint prefix may have been passed
        #: to the route
        if 'endpoint' in kwargs:
            endpoint = kwargs['endpoint'] + endpoint

        try:
            app.add_url_rule(
                url,
                view_func=self.view.as_view(endpoint),
                **self.kwargs)
        except AssertionError:
            # TODO: Log / Warn
            pass


class Blueprint(BaseRouter, RoutesImporter):
    """ Registers a flask blueprint and registers routes to that blueprint,
    similar to :py:class:`flask_via.routers.Include`.

    .. versionadded:: 2014.05.06

    Example
    -------

    **Auto creates Blueprint instance***

    .. sourcecode:: python

        from flask.ext.via.routers import default

        routes = [
            default.Blueprint('foo', 'flask_via.examples.blueprints.foo')
        ]

    **Pass existing Blueprint instance***

    .. sourcecode:: python

        from flask import Blueprint
        from flask.ext.via.routers import default

        blueprint = Blueprint('foo', __name__)

        routes = [
            default.Blueprint(blueprint)
        ]

    """

    def __init__(
            self,
            name_or_instance,
            module=None,
            routes_module_name='routes',
            routes_name=None,
            static_folder=None,
            static_url_path=None,
            template_folder=None,
            url_prefix=None,
            subdomain=None,
            url_defaults=None):
        """ Constructor for blueprint router.

        .. versionchanged:: 2014.05.19

            * Replaced ``name`` with ``name_or_instance`` argument which allows
              the router to take an already instantiated blueprint instance.
            * ``module`` argument optional when instance is passed as the
              first argument
            * ``routes_name`` keyword argument default value set to ``None``

        Arguments
        ---------
        name : str, flask.blueprints.Blueprint
            Blueprint name or a Blueprint class instance

        Keyword Arguments
        -----------------
        module : str
            Python dotted path to the blueprint module
        routes_module_name : str, optional
            The module ``Flask-Via`` will look for within the blueprint module
            which contains the routes, defaults to ``routes``
        routes_name : str, optional
            Name of the variable holding the routes in the module, defaults to
            ``None``
        static_folder : str, optional
            Path to static files for blueprint, defaults to ``None``
        static_url_path : str, optional
            URL path for blueprint static files, defaults to ``None``
        template_folder : str, optional
            Templates folder name, defaults to ``None``
        url_prefix : str, optional
            URL prefix for routes served within the blueprint, defaults
            to ``None``
        subdomain : str, optional
            Sub domain for blueprint, defaults to ``None``
        url_defaults : function, optional
            Callback function for URL defaults for this blueprint.
            It's called with the endpoint and values and should update
            the values passed in place, defaults to ``None``.
        """

        if isinstance(name_or_instance, FlaskBlueprint):
            self.instance = name_or_instance
            self.endpoint = self.instance.name
            self.module = self.instance.import_name
        else:
            self.endpoint = name_or_instance
            self.module = module

        self.routes_module_name = routes_module_name
        self.routes_name = routes_name
        self.static_folder = static_folder
        self.static_url_path = static_url_path
        self.template_folder = template_folder
        self.url_prefix = url_prefix
        self.subdomain = subdomain
        self.url_defaults = url_defaults

    @property
    def routes_module(self):
        """ Generates the routes module path, this is built from
        ``self.module`` and ``self.routes_module_name``.

        Returns
        -------
        str
            Python dotted path to the routes module containing routes.
        """

        return '{0}.{1}'.format(self.module, self.routes_module_name)

    def blueprint(self, **kwargs):
        """ Returns a Flask Blueprint instance, either one provided or created
        here.

        .. versionchanged:: 2014.05.19

            * Renamed method from ``create_blueprint`` to ``blueprint``
            * If ``instance`` attribute exists, use this is as the blueprint
              else create the blueprint.
            * Support for endpoint prefixing

        Returns
        -------
        flask.blueprints.Blueprint
            An instantiated Flask Blueprint instance
        """

        try:
            blueprint = self.instance
        except AttributeError:

            #: If this route was included a url preifx may have been passed
            #: to the route
            if 'url_prefix' in kwargs:
                url_prefix = self.url_prefix or ''
                self.url_prefix = kwargs['url_prefix'] + url_prefix

            #: If this route was included a endpoint prefix may have been
            #: passed to the route
            if 'endpoint' in kwargs:
                endpoint = self.endpoint or ''
                self.endpoint = kwargs['endpoint'] + endpoint

            blueprint = FlaskBlueprint(
                self.endpoint,
                self.module,
                static_folder=self.static_folder,
                static_url_path=self.static_url_path,
                template_folder=self.template_folder,
                url_prefix=self.url_prefix,
                subdomain=self.subdomain,
                url_defaults=self.url_defaults)

        return blueprint

    def add_to_app(self, app, **kwargs):
        """ Creates a Flask blueprint and registers routes with that blueprint,
        this means any routes defined will be added to the blueprint rather
        than the application.

        Arguments
        ---------
        app : flask.app.Flask
            Flask application instance
        \*\*kwargs
            Arbitrary keyword arguments passed in to ``init_app``
        """

        # Register blueproiint
        blueprint = self.blueprint(**kwargs)

        # Routes name can be configured by setting VIA_ROUTES_NAME
        if not self.routes_name:
            self.routes_name = app.config.get('VIA_ROUTES_NAME', 'routes')

        # Get the routes
        routes = self.include(self.routes_module, self.routes_name)

        # Load the routes
        self.load(blueprint, routes, **kwargs)

        # Register the blueprint with the application
        app.register_blueprint(blueprint)



# -*- coding: utf-8 -*-

"""
tests.test_routers.test_restful
================================

Unit tests for the Flask-Restful resource router.
"""

import mock
import unittest

from flask_via.routers import restful


class TestRestfulRouter(unittest.TestCase):

    def setUp(self):
        self.app = mock.MagicMock()

    def test_add_to_app_raises_not_implemented(self):

        resource = restful.Resource('/', mock.MagicMock())

        with self.assertRaises(NotImplementedError) as e:
            resource.add_to_app(self.app)

        self.assertEqual(
            str(e.exception),
            'restful_api not passed to add_to_app, did you add it to '
            'via.init_app?')

    def test_add_to_app(self):

        api = mock.MagicMock()

        class Resource(mock.MagicMock):
            pass

        resource = restful.Resource('/', Resource)
        resource.add_to_app(self.app, restful_api=api)

        api.add_resource.assert_called_once_with(Resource, '/', endpoint=None)



# -*- coding: utf-8 -*-

"""
flask_via
---------
"""

from flask_via.exceptions import ImproperlyConfigured
from importlib import import_module


class RoutesImporter(object):
    """ Handles the import of routes module and obtaining a list of routes
    from that module as well as loading each route onto the application

    .. versionadded:: 2014.05.06
    """

    def include(self, routes_module, routes_name):
        """ Imports a routes module and gets the routes from within that
        module and returns them.

        Arguments
        ---------
        routes_module : str
            Python dotted path to routes module
        routes_name : str
            Module attribute name to use when attempted to get the routes

        Returns
        -------
        list
            List of routes in the module

        Raises
        ------
        ImportError
            If the route module cannot be imported
        AttributeError
            If routes do not exist in the moduke
        """

        # Import the moduke
        module = import_module(routes_module)

        # Get the routes from the module
        routes = getattr(module, routes_name)

        return routes

    def load(self, app, routes, **kwargs):
        """ Loads passed routes onto the application by calling each routes
        ``add_to_app`` method which must be implemented by the route class.
        """

        for route in routes:
            route.add_to_app(app, **kwargs)


class Via(RoutesImporter):
    """ Flask-VIa integration into Flask applications. Flask-Via can
    be integrated in two different ways depending on how you have setup your
    Flask application.

    .. versionadded:: 2014.05.06

    You can bind to a specific flask application::

        from flask import Flask
        from flask.ext.via import Via
        from flask.ext.via.routers.flask import Functional

        app = Flask(__name__)

        def foo(bar=None):
            return 'Foo View!'

        routes = [
            Functional('/foo', foo),
            Functional('/foo/<bar>', foo, endpoint='foo2'),
        ]

        via = Via(app, routes_module='path.to.here')

        if __name__ == "__main__":
            app.run(debug=True)

    Or if you use an application factory you can use
    :meth:`flask_via.Via.init_app`::

        from flask import Flask
        from flask.ext.via import Via
        from flask.ext.via.routers.flask import Functional

        via = Via()

        def foo(bar=None):
            return 'Foo View!'

        routes = [
            Functional('/foo', foo),
            Functional('/foo/<bar>', foo, endpoint='foo2'),
        ]

        def create_app():
            app = Flask(__name__)
            via.init_app(app)
            return app

        app = create_app()

        if __name__ == "__main__":
            app.run(debug=True)
    """

    def __init__(self, app=None, *args, **kwargs):
        """ Constructor. Functionalally acts as a proxy to
        :meth:`flask_store.Store.init_app`.

        .. versionadded:: 2014.05.19.2

        Key Arguments
        -------------
        app : flask.app.Flask, optional
            Optional Flask application instance, default None
        """

        if app:
            self.init_app(app, *args, **kwargs)

    def init_app(
            self,
            app,
            routes_module=None,
            routes_name=None,
            **kwargs):
        """ Initialises Flask extension. Bootstraps the automatic route
        registration process.

        .. versionchanged:: 2014.05.19

            * Replace ``NotImplementedError`` with ``ImproperlyConfigured``
            * ``routes_name`` keyword argument default value set to ``None``
            * ``routes_name`` can now be configured using ``VIA_ROUTES_NAME``
              app configuration variable. If ``routes_name`` keyword argument
              and ``VIA_ROUTES_NAME`` are not configured the default will be
              routes.

        .. versionchanged:: 2014.05.19.2

            * Improved ``init_app`` method

        Arguments
        ---------
        app : flask.app.Flask
            Flask application instance

        Keyword Arguments
        -----------------
        route_module : str, optional
            Python dotted path to where routes are defined, defaults
            to ``None``
        routes_name : str, optional
            Within the routes module look for a variable of this name,
            defaults to ``None``
        \*\*kwargs
            Arbitrary keyword arguments passed to ``add_url_rule``

        Raises
        ------
        ImproperlyConfigured
            If ``VIA_ROUTES_MODULE`` is not configured in appluication config
            and ``route_module`` keyword argument has not been provided.
        """

        app.config.setdefault('VIA_ROUTES_MODULE', routes_module)
        app.config.setdefault('VIA_ROUTES_NAME', routes_name or 'routes')

        if not app.config['VIA_ROUTES_MODULE']:
            raise ImproperlyConfigured(
                'VIA_ROUTES_MODULE is not defined in application '
                'configuration.')

        routes_module = app.config['VIA_ROUTES_MODULE']
        routes_name = app.config['VIA_ROUTES_NAME']

        # Get the routes
        routes = self.include(routes_module, routes_name)

        # Load the routes
        self.load(app, routes, **kwargs)



# -*- coding: utf-8 -*-

"""
flask_via.examples.small.views
==============================

A small ``Flask-Via`` example Flask application.
"""


def home():
    return 'Home Page'


def contact():
    return 'Contact Page'


def about():
    return 'About Page'



# -*- coding: utf-8 -*-

"""
flask_via.examples.include.foo.routes
=====================================

A include ``Flask-Via`` example Flask application.
"""

from flask_via.examples.include.bar.views import FooView, FazView, flop
from flask.ext.via.routers.default import Functional, Pluggable

routes = [
    Pluggable('/foo', FooView, 'foo'),
    Pluggable('/faz', FazView, 'faz'),
    Functional('/flop', flop),
]



# -*- coding: utf-8 -*-

"""
flask_via.examples.small.routes
===============================

A small ``Flask-Via`` example Flask application.
"""

from flask_via.examples.small import views
from flask.ext.via.routers import default, Include

routes = [
    default.Functional('/', views.home),
    default.Functional('/about', views.about),
    default.Functional('/contact', views.contact),
    # Include other routes from other modules
    Include('flask_via.examples.small.foo.routes', url_prefix='/foo')
]



# -*- coding: utf-8 -*-

"""
flask_via.routers.admin
-----------------------

Routers for the Flask-Admin framework.
"""

from flask_via.routers import BaseRouter


class AdminRoute(BaseRouter):
    """ The Admin router allows you to define Flask-Admin routes and have
    those views added to the application automatically. For this to
    work you must at ``init_app`` time pass a optional keyword argument
    ``flask_admin`` to ``init_app`` with its value being the Flask-Aadmin
    extension instance.

    .. versionadded:: 2014.05.08

    Note
    ----
    ``Flask-Admin`` has its own way of handling defining urls so this router
    literally only requires the ``Flask-Admin`` view class.

    Example
    -------
    .. sourcecode:: python

        app = Flask(__name__)

        admin = Admin(name='Admin')
        admin.init_app(app)


        class FooAdminView(BaseView):

            @expose('/')
            def index(self):
                return 'foo'

        routes = [
            AdminRoute(FooAdminView(name='Foo'))
        ]

        via = Via()
        via.init_app(
            app,
            routes_module='flask_via.examples.admin',
            flask_admin=admin)

        if __name__ == '__main__':
            app.run(debug=True)
    """

    def __init__(self, view):
        """ Admin route constructor, this router handles adding ``Flask-admin``
        views to the application.

        Arguments
        ---------
        view : flask_admin.base.AdminViewMeta
            The Flask Admin View Class
        """

        self.view = view

    def add_to_app(self, app, **kwargs):
        """ Adds the ``Flask-Admin`` View to the Flask the application.

        Arguments
        ---------
        app : flask.app.Flask
            Flask application instance, this is ignored.
        \*\*kwargs
            Arbitrary keyword arguments

        Raises
        ------
        NotImplementedError
            If ``flask_admin`` is not provided
        """

        try:
            admin = kwargs['flask_admin']
        except KeyError:
            raise NotImplementedError(
                'flask_admin not passed to add_to_app, did you add it to '
                'via.init_app?')

        admin.add_view(self.view)



# -*- coding: utf-8 -*-

"""
flask_via.examples.mixed
========================

Mixed router example.
"""

from flask import Flask
from flask.views import MethodView
from flask.ext import restful
from flask.ext.via import Via
from flask.ext.via.routers.default import Functional, Pluggable
from flask.ext.via.routers.restful import Resource


class FooView(MethodView):

    def get(self, bar=None):
        rtn = 'Pluggable Foo View'
        if bar:
            rtn += ' bar = {0}'.format(bar)

        return rtn


class FooResource(restful.Resource):

    def get(self, bar=None):
        rtn = {'resource': 'foo'}
        if bar:
            rtn['bar'] = bar

        return rtn


def foo(bar=None):
    rtn = 'Functional Foo View'
    if bar:
        rtn += ' bar = {0}'.format(bar)

    return rtn


app = Flask(__name__)
api = restful.Api(app)

routes = [
    # Functional Views
    Functional('/basic', foo),
    Functional('/basic/<bar>', foo, endpoint='foo.basic'),
    # Pluggable Viewa
    Pluggable('/pluggable', FooView, 'foo.pluggable'),
    Pluggable('/pluggable/<bar>', FooView, 'foobar.pluggable'),
    # Flask Restful Resource Views
    Resource('/resource', FooResource, 'foo.resource'),
    Resource('/resource/<bar>', FooResource, endpoint='foobar.resource')
]

via = Via()
via.init_app(
    app,
    routes_module='flask_via.examples.mixed',
    restful_api=api)

if __name__ == '__main__':
    app.run(debug=True)



# -*- coding: utf-8 -*-

"""
flask_via.examples.blueprints.foo.routes
========================================

A blueprint ``Flask-Via`` example Flask application.
"""

from flask_via.examples.blueprints.foo import views
from flask.ext.via.routers import default

routes = [
    default.Functional('/baz', views.baz),
]



# -*- coding: utf-8 -*-

"""
flask_via.examples.blueprints.routes
====================================

A blueprint ``Flask-Via`` example Flask application.
"""

from flask_via.examples.blueprints.baz import blueprint
from flask.ext.via.routers import default, Include

routes = [
    # Create the blueprint instance
    default.Blueprint(
        'foo',
        'flask_via.examples.blueprints.foo',
        url_prefix='/foo',
        template_folder='templates'
    ),
    # Use a blueprint instance
    default.Blueprint(blueprint),
    # Include blueprints
    Include(
        'flask_via.examples.blueprints.routes',
        routes_name='urls',
        url_prefix='/bar',
        endpoint='bar'
    )
]

urls = [
    default.Blueprint(
        'foo',
        'flask_via.examples.blueprints.foo',
        url_prefix='/foo',
        template_folder='templates'
    )
]



# -*- coding: utf-8 -*-

"""
flask_via.examples.blueprints.baz
=================================

A blueprint ``Flask-Via`` example Flask application.
"""

from flask import Blueprint


blueprint = Blueprint(
    'baz', __name__, template_folder='templates', url_prefix='/baz')



# -*- coding: utf-8 -*-

"""
flask_via.examples.blueprints.app
=================================

A blueprint ``Flask-Via`` example Flask application.
"""

from flask import Flask
from flask.ext.via import Via

app = Flask(__name__)
app.config['VIA_ROUTES_MODULE'] = 'flask_via.examples.blueprints.routes'

via = Via()
via.init_app(app)

if __name__ == '__main__':
    app.run(debug=True)



# -*- coding: utf-8 -*-

"""
flask_via.examples.blueprints.baz.views
=======================================

A blueprint ``Flask-Via`` example Flask application.
"""

from flask import render_template


def baz():
    return render_template('baz.html')



# -*- coding: utf-8 -*-

"""
flask_via.examples.include.routes
=================================

A include ``Flask-Via`` example Flask application.
"""

from flask.ext.via.routers import Include

routes = [
    Include(
        'flask_via.examples.include.foo.routes',
        url_prefix='/foo',
        endpoint='foo')
]



# -*- coding: utf-8 -*-

"""
tests.test_routers.test_default
===============================

Unit tests for Flask specific router classes.
"""

import mock

from flask import Blueprint, url_for
from flask.views import MethodView
from flask_via.routers import default
from tests import ViaTestCase


class TestFlaskFunctionalRouter(ViaTestCase):

    def setUp(self):
        self.view = mock.MagicMock(
            __name__='foo',
            methods=['GET', ],
            return_value='foo')

    def test_add_to_app(self):
        route = default.Functional('/', self.view, endpoint='foo')
        route.add_to_app(self.app)

        self.assertEqual(url_for('foo'), '/')
        self.assertEqual(self.client.get('/').data, b'foo')

    def test_url_prefix(self):
        route = default.Functional('/', self.view, endpoint='foo')
        route.add_to_app(self.app, url_prefix='/foo')

        self.assertEqual(url_for('foo'), '/foo/')
        self.assertEqual(self.client.get('/foo/').data, b'foo')

    def test_endpoint_prefix(self):
        route = default.Functional('/', self.view, endpoint='foo')
        route.add_to_app(self.app, endpoint='bar.')

        self.assertEqual(url_for('bar.foo'), '/')
        self.assertEqual(self.client.get('/').data, b'foo')

    def test_default_endpoint_name(self):
        route = default.Functional('/', self.view)
        route.add_to_app(self.app, endpoint='bar.')

        self.assertEqual(url_for('bar.foo'), '/')
        self.assertEqual(self.client.get('/').data, b'foo')


class TestFlaskPluggableRouter(ViaTestCase):

    def setUp(self):
        class View(MethodView):

            def get(self):
                return 'foo'

        self.View = View

    def test_add_to_app(self):
        route = default.Pluggable('/', self.View, 'foo')
        route.add_to_app(self.app)

        self.assertEqual(url_for('foo'), '/')
        self.assertEqual(self.client.get('/').data, b'foo')

    def test_url_prefix(self):
        route = default.Pluggable('/', self.View, 'foo')
        route.add_to_app(self.app, url_prefix='/foo')

        self.assertEqual(url_for('foo'), '/foo/')
        self.assertEqual(self.client.get('/foo/').data, b'foo')

    def test_endpoint_prefix(self):
        route = default.Pluggable('/', self.View, endpoint='foo')
        route.add_to_app(self.app, endpoint='bar.')

        self.assertEqual(url_for('bar.foo'), '/')
        self.assertEqual(self.client.get('/').data, b'foo')


class TestBlueprintRouter(ViaTestCase):

    def test_routes_module_path(self):
        route = default.Blueprint('foo', 'foo.bar')

        self.assertEqual(route.routes_module, 'foo.bar.routes')

    @mock.patch('flask.helpers.get_root_path')
    def test_bluepint_creates_blurprint(self, _get_root_path):
        route = default.Blueprint('foo', 'foo.bar')

        self.assertIsInstance(route.blueprint(), Blueprint)

    @mock.patch('flask.helpers.get_root_path')
    def test_bluepint_returns_blurprint(self, _get_root_path):
        blueprint = Blueprint('foo', __name__)
        route = default.Blueprint(blueprint)

        self.assertIsInstance(route.blueprint(), Blueprint)
        self.assertEqual(route.blueprint(), blueprint)

    @mock.patch('flask.helpers.get_root_path')
    def test_url_prefix(self, _get_root_path):
        route = default.Blueprint('foo', 'foo.bar')
        blueprint = route.blueprint(url_prefix='/foo')

        self.assertEqual(blueprint.url_prefix, '/foo')

    @mock.patch('flask.helpers.get_root_path')
    def test_endpoint_prefix(self, _get_root_path):
        route = default.Blueprint('bar', 'foo.bar')
        blueprint = route.blueprint(endpoint='foo.')

        self.assertEqual(blueprint.name, 'foo.bar')

    @mock.patch('flask_via.import_module')
    def test_add_to_app(self, _import_module):
        foo = mock.MagicMock(
            __name__='foo',
            methods=['GET', ],
            return_value='foo')
        bar = mock.MagicMock(
            __name__='bar',
            methods=['GET', ],
            return_value='bar')

        routes = [
            default.Functional('/foo', foo, 'foo'),
            default.Functional('/bar', bar, 'bar'),
        ]

        _import_module.side_effect = [
            mock.MagicMock(routes=routes),
        ]

        route = default.Blueprint('foo', 'foo.bar')

        with mock.patch('flask.helpers.pkgutil.get_loader'):
            route.add_to_app(self.app)

        self.assertEqual(url_for('foo.foo'), '/foo')
        self.assertEqual(url_for('foo.bar'), '/bar')



# -*- coding: utf-8 -*-

"""
flask_via.examples.admin
========================

A simple ``Flask-Admin`` example Flask application.
"""

from flask import Flask
from flask.ext.admin import Admin
from flask.ext.via import Via
from flask_admin import BaseView, expose
from flask.ext.via.routers.admin import AdminRoute


app = Flask(__name__)

admin = Admin(name='Admin')
admin.init_app(app)


class FooAdminView(BaseView):

    @expose('/')
    def index(self):
        return 'foo'

routes = [
    AdminRoute(FooAdminView(name='Foo'))
]

via = Via()
via.init_app(
    app,
    routes_module='flask_via.examples.admin',
    flask_admin=admin)

if __name__ == '__main__':
    app.run(debug=True)



# -*- coding: utf-8 -*-

"""
flask_via.examples.include.bar.views
====================================

A include ``Flask-Via`` example Flask application.
"""

from flask import request
from flask.views import MethodView


class FooView(MethodView):

    def get(self):
        return '{0.path} - {0.endpoint}'.format(request)


class FazView(MethodView):

    def get(self):
        return '{0.path} - {0.endpoint}'.format(request)


def flop():
    return '{0.path} - {0.endpoint}'.format(request)



# -*- coding: utf-8 -*-

"""
flask_via.examples.restful
==========================

A simple ``Flask-Via`` example Flask application.
"""

from flask import Flask
from flask.ext import restful
from flask.ext.via import Via
from flask.ext.via.routers.restful import Resource


app = Flask(__name__)
api = restful.Api(app)


class FooResource(restful.Resource):

    def get(self, bar=None):
        rtn = {'hello': 'world'}
        if bar:
            rtn['bar'] = bar

        return rtn


routes = [
    Resource('/', FooResource),
    Resource('/<bar>', FooResource, endpoint='foobar')
]

via = Via()
via.init_app(
    app,
    routes_module='flask_via.examples.restful',
    restful_api=api)


if __name__ == '__main__':
    app.run(debug=True)



# -*- coding: utf-8 -*-

"""
flask_via.examples.basic
========================

A simple ``Flask-Via`` example Flask application.
"""

from flask import Flask
from flask.ext.via import Via
from flask.ext.via.routers.default import Functional


app = Flask(__name__)


def foo(bar=None):
    return 'Functional Foo View!'


routes = [
    Functional('/foo', foo),
    Functional('/foo/<bar>', foo, endpoint='foo2'),
]

via = Via()
via.init_app(app, routes_module='flask_via.examples.basic')

if __name__ == "__main__":
    app.run(debug=True)



# -*- coding: utf-8 -*-

"""
flask_via.examples.pluggable
============================

A simple ``Flask-Via`` example Flask application.
"""

from flask import Flask
from flask.views import MethodView
from flask.ext.via import Via
from flask.ext.via.routers.default import Pluggable


class FooView(MethodView):

    def get(self, bar=None):
        if bar:
            return 'Pluggable - Foo View with Bar = {0}'.format(bar)
        return 'Pluggable - Foo View'

app = Flask(__name__)

routes = [
    Pluggable('/foo', FooView, 'foo'),
    Pluggable('/foo/<bar>', FooView, 'foo2'),
]

via = Via()
via.init_app(app, routes_module='flask_via.examples.pluggable')

if __name__ == "__main__":
    app.run(debug=True)



# -*- coding: utf-8 -*-

"""
tests.test_routers.test_admin
=============================

Unit tests for the Flask-Admin admin router.
"""

import mock
import unittest

from flask_via.routers import admin


class TestRestfulRouter(unittest.TestCase):

    def setUp(self):
        self.app = mock.MagicMock()

    def test_add_to_app_raises_not_implemented(self):

        resource = admin.AdminRoute(mock.MagicMock())

        with self.assertRaises(NotImplementedError) as e:
            resource.add_to_app(self.app)

        self.assertEqual(
            str(e.exception),
            'flask_admin not passed to add_to_app, did you add it to '
            'via.init_app?')

    def test_add_to_app(self):

        flask_admin = mock.MagicMock()

        class FooAdminView(mock.MagicMock):
            pass

        route = admin.AdminRoute(FooAdminView)
        route.add_to_app(self.app, flask_admin=flask_admin)

        flask_admin.add_view.assert_called_once_with(FooAdminView)



# -*- coding: utf-8 -*-
#!/usr/bin/env python

"""
Flask-Via
---------

Flask-Via is a url routing register designed for larger Flask applications
allowing developers to much more cleanly manage routes across your
application.
"""

# Temporary patch for issue reported here:
# https://groups.google.com/forum/#!topic/nose-users/fnJ-kAUbYHQ
import multiprocessing  # noqa
import os
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


def read_requirements(filename):
    """ Read requirements file and process them into a list
    for usage in the setup function.

    Arguments
    ---------
    filename : str
        Path to the file to read line by line

    Returns
    --------
    list
        list of requirements::

            ['package==1.0', 'thing>=9.0']
    """

    requirements = []
    with open(filename) as f:
        for line in f.readlines():
            if not line or line.startswith('#'):
                continue
            requirements.append(line.strip())
    return requirements

# Get current working directory

try:
    SETUP_DIRNAME = os.path.dirname(__file__)
except NameError:
    SETUP_DIRNAME = os.path.dirname(sys.argv[0])

# Change to current working directory

if SETUP_DIRNAME != '':
    os.chdir(SETUP_DIRNAME)

# Requirements

INSTALL_REQUIREMENTS = read_requirements('REQS.txt')
TESTING_REQUIREMENTS = read_requirements('REQS.TESTING.txt')
DEVELOP_REQUIREMENTS = read_requirements('REQS.DEVELOP.txt') \
    + TESTING_REQUIREMENTS

# Include the Change Log on PyPi

long_description = open('README.rst').read()
changelog = open('CHANGELOG.rst').read()
long_description += '\n' + changelog

# Setup

setup(
    name='Flask-Via',
    version=open('VERSION').read().strip(),
    author='SOON_',
    author_email='dorks@thisissoon.com',
    maintainer='Chris Reeves',
    maintainer_email='hello@chris.reeves.io',
    url='http://flask-via.soon.build',
    description='Provides a clean, simple URL routing framework for '
                'growing Flask Applications.',
    long_description=long_description,
    packages=find_packages(
        exclude=[
            'tests'
        ]),
    include_package_data=True,
    zip_safe=False,
    # Dependencies
    install_requires=INSTALL_REQUIREMENTS,
    extras_require={
        'develop': DEVELOP_REQUIREMENTS
    },
    # Testing
    tests_require=TESTING_REQUIREMENTS,
    cmdclass={
        'test': PyTest
    },
    # Dependencies not hosted on PyPi
    dependency_links=[],
    # Classifiers for Package Indexing
    # Entry points, for example Flask-Script
    entry_points={},
    # Meta
    classifiers=[
        'Framework :: Flask',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content'],
    license='MIT',
    keywords=['Flask', 'Routing', 'Routes', 'URLs', 'Views'])



# -*- coding: utf-8 -*-

"""
flask_via.examples.small.foo.routes
===================================

A small ``Flask-Via`` example Flask application.
"""

from flask_via.examples.small.foo import views
from flask.ext.via.routers import default, Include

routes = [
    default.Functional('/foo', views.foo),
    Include('flask_via.examples.small.foo.routes',
            routes_name='urls',
            url_prefix='/bar')
]

urls = [
    default.Functional('/bar', views.foo),
]



# -*- coding: utf-8 -*-

"""
flask_via.routers
-----------------

Base router classes and utilities.
"""

from flask_via import RoutesImporter


class BaseRouter(object):
    """ Base router class all routers should inherit from providing common
    router functionality.

    .. versionadded:: 2014.05.06

    Example
    -------
    .. sourcecode:: python

        from flask.ext.via.routers import BaseRouter

        class MyRouter(BaseRouter):

            def __init__(self, arg):
                ...

            def add_to_app(self, app):
                ...
    """

    def __init__(self):
        """ Constructor should be overridden to accept specific arguments
        for the router.

        Raises
        ------
        NotImplementedError
            If method not implemented
        """

        raise NotImplementedError('__init__ must be overridden')

    def add_to_app(self, app, **kwargs):
        """ Method all routers require, which handles adding the route to
        the application instance.

        Raises
        ------
        NotImplementedError
            If method not implemented
        """

        raise NotImplementedError('add_to_app must be overridden')


class Include(BaseRouter, RoutesImporter):
    """ Adds the ability to include routes from other modules, this can be
    handy when you want to break out your routes into separate files for
    sanity.

    .. versionadded:: 2014.05.06

    Note
    ----
    This is not a implementation of Flask blueprints
    """

    def __init__(
            self,
            routes_module,
            routes_name=None,
            url_prefix=None,
            endpoint=None):
        """ Constructor for Include router, taking the passed arguments
        and storing them on the instance.

        .. versionchanged:: 2014.05.08

            * ``url_prefix`` argument added

        .. versionchanged:: 2014.05.19

            * ``routes_name`` keyword argument default value set to ``None``
            * ``endpoint`` keyword argument added

        Arguments
        ---------
        routes_module : str
            Python dotted path to the routes module

        Keyword Arguments
        -----------------
        routes_name : str, optional
            Name of the variable holding the routes in the module, defaults to
            ``routes``
        url_prefix : str, optional
            Adds a url prefix to all routes included by the router, defaults
            to ``None``
        endpoint : str, optional
            Prefix an endpoint to all routes included, defaults to ``None``
        """

        self.routes_module = routes_module
        self.routes_name = routes_name
        self.url_prefix = url_prefix
        self.endpoint = endpoint

    def add_to_app(self, app, **kwargs):
        """ Instead of adding a route to the flask application this will
        include and load routes similar, same as in the
        :py:class:`flask_via.Via` class.abs

        .. versionchanged:: 2014.05.08

            * ``url_prefix`` now injected into kwargs when loading in routes

        .. versionchanged:: 2014.05.19

            * ``endpoint`` now injects into kwargs when loading in routes

        Arguments
        ---------
        app : flask.app.Flask
            Flask application instance
        \*\*kwargs
            Arbitrary keyword arguments passed in to ``init_app``
        """

        # Routes name can be configured by setting VIA_ROUTES_NAME
        if not self.routes_name:
            self.routes_name = app.config.get('VIA_ROUTES_NAME', 'routes')

        # Inject url_prefix into kwargs
        if self.url_prefix:
            # This allows us to chain url prefix's when multiple includes
            # are called
            try:
                url_prefix = kwargs['url_prefix']
            except KeyError:
                url_prefix = ''
            finally:
                kwargs['url_prefix'] = url_prefix + self.url_prefix

        # Inject endpoint into kwagrs
        if self.endpoint is not None:
            # This allows us to add a endpoint prefix to routes included
            try:
                endpoint = kwargs['endpoint']
            except KeyError:
                endpoint = ''
            finally:
                kwargs['endpoint'] = endpoint + self.endpoint + '.'

        # Get the routes
        routes = self.include(self.routes_module, self.routes_name)

        # Load the routes
        self.load(app, routes, **kwargs)



# -*- coding: utf-8 -*-

"""
tests.test_routers
==================

Unit tests for common router classes and utilities.
"""

import copy
import mock
import unittest

from flask import url_for
from flask_via.routers import BaseRouter, Include
from flask_via.routers.default import Functional
from tests import ViaTestCase


class TestBaseRouter(unittest.TestCase):

    def test_init_must_be_implemented(self):

        class FooRouter(BaseRouter):
            pass

        with self.assertRaises(NotImplementedError) as e:
            FooRouter()

        self.assertTrue(str(e.exception), '__init__ must be overridden')


class TestIncludeRouter(ViaTestCase):

    def setUp(self):
        self.foo_view = mock.MagicMock(
            __name__='foo',
            methods=['GET', ],
            return_value='foo')
        self.bar_view = mock.MagicMock(
            __name__='bar',
            methods=['GET', ],
            return_value='bar')
        self.routes = [
            Functional('/foo', self.foo_view, 'foo'),
            Functional('/bar', self.bar_view, 'bar'),
        ]

    def test_init(self):
        route = Include('foo.bar', routes_name='urls')

        self.assertEqual(route.routes_module, 'foo.bar')
        self.assertEqual(route.routes_name, 'urls')

    @mock.patch('flask_via.import_module')
    def test_add_to_app(self, _import_module):
        _import_module.return_value = mock.MagicMock(urls=self.routes)

        route = Include('foo.bar', routes_name='urls')
        route.add_to_app(self.app)

        self.assertEqual(url_for('foo'), '/foo')
        self.assertEqual(url_for('bar'), '/bar')

    @mock.patch('flask_via.import_module')
    def test_url_prefix(self, _import_module):
        routes1 = copy.deepcopy(self.routes)
        routes1.append(Include(
            'foo.bar',
            url_prefix='/prefix2',
            endpoint='prefix2'))
        routes2 = copy.deepcopy(self.routes)

        _import_module.side_effect = [
            mock.MagicMock(routes=routes1),
            mock.MagicMock(routes=routes2)
        ]

        include = Include('foo.bar', url_prefix='/prefix1')
        include.add_to_app(self.app)

        self.assertEqual(url_for('foo'), '/prefix1/foo')
        self.assertEqual(url_for('bar'), '/prefix1/bar')
        self.assertEqual(url_for('prefix2.foo'), '/prefix1/prefix2/foo')
        self.assertEqual(url_for('prefix2.bar'), '/prefix1/prefix2/bar')

    @mock.patch('flask_via.import_module')
    def test_endpoint_prefix(self, _import_module):
        routes1 = copy.deepcopy(self.routes)
        routes1.append(Include(
            'foo.bar',
            endpoint='endpoint2'))
        routes2 = copy.deepcopy(self.routes)

        _import_module.side_effect = [
            mock.MagicMock(routes=routes1),
            mock.MagicMock(routes=routes2)
        ]

        include = Include('foo.bar', endpoint='endpoint1')
        include.add_to_app(self.app)

        self.assertEqual(url_for('endpoint1.foo'), '/foo')
        self.assertEqual(url_for('endpoint1.bar'), '/bar')
        self.assertEqual(url_for('endpoint1.endpoint2.foo'), '/foo')
        self.assertEqual(url_for('endpoint1.endpoint2.bar'), '/bar')
